@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -48,12 +48,20 @@ class Option:
     backends = attr.ib()
     raw_backends = attr.ib()
     description = attr.ib()
+    supports_pattern = attr.ib(default=False)
+    restart = attr.ib(default=False)
+    no_autoconfig = attr.ib(default=False)
 
 
 @attr.s
 class Migrations:
 
-    """Nigrated options in configdata.yml."""
+    """Nigrated options in configdata.yml.
+
+    Attributes:
+        renamed: A dict mapping old option names to new names.
+        deleted: A list of option names which have been removed.
+    """
 
     renamed = attr.ib(default=attr.Factory(dict))
     deleted = attr.ib(default=attr.Factory(list))
@@ -173,6 +181,7 @@ def _parse_yaml_backends(name, node):
     elif isinstance(node, dict):
         return _parse_yaml_backends_dict(name, node)
     _raise_invalid_node(name, 'backends', node)
+    raise utils.Unreachable
 
 
 def _read_yaml(yaml_data):
@@ -190,7 +199,8 @@ def _read_yaml(yaml_data):
     migrations = Migrations()
     data = utils.yaml_load(yaml_data)
 
-    keys = {'type', 'default', 'desc', 'backend'}
+    keys = {'type', 'default', 'desc', 'backend', 'restart',
+            'supports_pattern', 'no_autoconfig'}
 
     for name, option in data.items():
         if set(option.keys()) == {'renamed'}:
@@ -215,7 +225,11 @@ def _read_yaml(yaml_data):
             default=option['default'],
             backends=_parse_yaml_backends(name, backends),
             raw_backends=backends if isinstance(backends, dict) else None,
-            description=option['desc'])
+            description=option['desc'],
+            restart=option.get('restart', False),
+            supports_pattern=option.get('supports_pattern', False),
+            no_autoconfig=option.get('no_autoconfig', False),
+        )
 
     # Make sure no key shadows another.
     for key1 in parsed:
