@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -25,8 +25,8 @@ from PyQt5.QtWidgets import (QLineEdit, QWidget, QHBoxLayout, QLabel,
 from PyQt5.QtGui import QValidator, QPainter
 
 from qutebrowser.config import config
-from qutebrowser.utils import utils, qtutils, log, usertypes
-from qutebrowser.misc import cmdhistory, objects
+from qutebrowser.utils import utils
+from qutebrowser.misc import cmdhistory
 
 
 class MinimalLineEditMixin:
@@ -241,7 +241,7 @@ class WrapperLayout(QLayout):
         self._widget = None
 
     def addItem(self, _widget):
-        raise AssertionError("Should never be called!")
+        raise utils.Unreachable
 
     def sizeHint(self):
         return self._widget.sizeHint()
@@ -250,7 +250,7 @@ class WrapperLayout(QLayout):
         return None
 
     def takeAt(self, _index):
-        raise AssertionError("Should never be called!")
+        raise utils.Unreachable
 
     def setGeometry(self, rect):
         self._widget.setGeometry(rect)
@@ -260,16 +260,6 @@ class WrapperLayout(QLayout):
         self._widget = widget
         container.setFocusProxy(widget)
         widget.setParent(container)
-        if (qtutils.version_check('5.8.0', exact=True, compiled=False) and
-                objects.backend == usertypes.Backend.QtWebEngine and
-                container.window() and
-                container.window().windowHandle() and
-                not container.window().windowHandle().isActive()):
-            log.misc.debug("Calling QApplication::sync...")
-            # WORKAROUND for:
-            # https://bugreports.qt.io/browse/QTBUG-56652
-            # https://codereview.qt-project.org/#/c/176113/5//ALL,unified
-            QApplication.sync()
 
     def unwrap(self):
         self._widget.setParent(None)
@@ -293,14 +283,15 @@ class FullscreenNotification(QLabel):
         bindings = all_bindings.get('fullscreen --leave')
         if bindings:
             key = bindings[0]
-            if utils.is_special_key(key):
-                key = key.strip('<>').capitalize()
             self.setText("Press {} to exit fullscreen.".format(key))
         else:
             self.setText("Page is now fullscreen.")
 
         self.resize(self.sizeHint())
-        geom = QApplication.desktop().screenGeometry(self)
+        if config.val.content.windowed_fullscreen:
+            geom = self.parentWidget().geometry()
+        else:
+            geom = QApplication.desktop().screenGeometry(self)
         self.move((geom.width() - self.sizeHint().width()) / 2, 30)
 
     def set_timeout(self, timeout):
