@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU General Public License
 # along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Utilities to show various version informations."""
+"""Utilities to show various version information."""
 
 import re
 import sys
@@ -27,9 +27,12 @@ import platform
 import subprocess
 import importlib
 import collections
-import pkg_resources
+import enum
+import datetime
+import getpass
 
 import attr
+import pkg_resources
 from PyQt5.QtCore import PYQT_VERSION_STR, QLibraryInfo
 from PyQt5.QtNetwork import QSslSocket
 from PyQt5.QtGui import (QOpenGLContext, QOpenGLVersionProfile,
@@ -47,8 +50,8 @@ except ImportError:  # pragma: no cover
     QWebEngineProfile = None
 
 import qutebrowser
-from qutebrowser.utils import log, utils, standarddir, usertypes
-from qutebrowser.misc import objects, earlyinit, sql
+from qutebrowser.utils import log, utils, standarddir, usertypes, message
+from qutebrowser.misc import objects, earlyinit, sql, httpclient, pastebin
 from qutebrowser.browser import pdfjs
 
 
@@ -63,7 +66,8 @@ class DistributionInfo:
     pretty = attr.ib()
 
 
-Distribution = usertypes.enum(
+pastebin_url = None
+Distribution = enum.Enum(
     'Distribution', ['unknown', 'ubuntu', 'debian', 'void', 'arch',
                      'gentoo', 'fedora', 'opensuse', 'linuxmint', 'manjaro'])
 
@@ -150,19 +154,22 @@ def _git_str_subprocess(gitpath):
     if not os.path.isdir(os.path.join(gitpath, ".git")):
         return None
     try:
-        cid = subprocess.check_output(
-            ['git', 'describe', '--tags', '--dirty', '--always'],
-            cwd=gitpath).decode('UTF-8').strip()
-        date = subprocess.check_output(
+        # https://stackoverflow.com/questions/21017300/21017394#21017394
+        commit_hash = subprocess.run(
+            ['git', 'describe', '--match=NeVeRmAtCh', '--always', '--dirty'],
+            cwd=gitpath, check=True,
+            stdout=subprocess.PIPE).stdout.decode('UTF-8').strip()
+        date = subprocess.run(
             ['git', 'show', '-s', '--format=%ci', 'HEAD'],
-            cwd=gitpath).decode('UTF-8').strip()
-        return '{} ({})'.format(cid, date)
+            cwd=gitpath, check=True,
+            stdout=subprocess.PIPE).stdout.decode('UTF-8').strip()
+        return '{} ({})'.format(commit_hash, date)
     except (subprocess.CalledProcessError, OSError):
         return None
 
 
 def _release_info():
-    """Try to gather distribution release informations.
+    """Try to gather distribution release information.
 
     Return:
         list of (filename, content) tuples.
@@ -298,7 +305,19 @@ def _pdfjs_version():
 
 
 def _chromium_version():
-    """Get the Chromium version for QtWebEngine."""
+    """Get the Chromium version for QtWebEngine.
+
+    This can also be checked by looking at this file with the right Qt tag:
+    https://github.com/qt/qtwebengine/blob/dev/tools/scripts/version_resolver.py#L41
+
+    Quick reference:
+    Qt 5.7:  Chromium 49
+    Qt 5.8:  Chromium 53
+    Qt 5.9:  Chromium 56
+    Qt 5.10: Chromium 61
+    Qt 5.11: Chromium 63
+    Qt 5.12: Chromium 65 (?)
+    """
     if QWebEngineProfile is None:
         # This should never happen
         return 'unavailable'
@@ -321,8 +340,16 @@ def _backend():
         return 'QtWebEngine (Chromium {})'.format(_chromium_version())
 
 
+def _uptime() -> datetime.timedelta:
+    launch_time = QApplication.instance().launch_time
+    time_delta = datetime.datetime.now() - launch_time
+    # Round off microseconds
+    time_delta -= datetime.timedelta(microseconds=time_delta.microseconds)
+    return time_delta
+
+
 def version():
-    """Return a string with various version informations."""
+    """Return a string with various version information."""
     lines = ["qutebrowser v{}".format(qutebrowser.__version__)]
     gitver = _git_str()
     if gitver is not None:
@@ -384,6 +411,11 @@ def version():
     for name, path in sorted(_path_info().items()):
         lines += ['{}: {}'.format(name, path)]
 
+    lines += [
+        '',
+        'Uptime: {}'.format(_uptime()),
+    ]
+
     return '\n'.join(lines)
 
 
@@ -431,3 +463,40 @@ def opengl_vendor():  # pragma: no cover
         ctx.doneCurrent()
         if old_context and old_surface:
             old_context.makeCurrent(old_surface)
+
+
+def pastebin_version(pbclient=None):
+    """Pastebin the version and log the url to messages."""
+    def _yank_url(url):
+        utils.set_clipboard(url)
+        message.info("Version url {} yanked to clipboard.".format(url))
+
+    def _on_paste_version_success(url):
+        global pastebin_url
+        _yank_url(url)
+        pbclient.deleteLater()
+        pastebin_url = url
+
+    def _on_paste_version_err(text):
+        message.error("Failed to pastebin version"
+                      " info: {}".format(text))
+        pbclient.deleteLater()
+
+    if pastebin_url:
+        _yank_url(pastebin_url)
+        return
+
+    app = QApplication.instance()
+    http_client = httpclient.HTTPClient()
+
+    misc_api = pastebin.PastebinClient.MISC_API_URL
+    pbclient = pbclient or pastebin.PastebinClient(http_client, parent=app,
+                                                   api_url=misc_api)
+
+    pbclient.success.connect(_on_paste_version_success)
+    pbclient.error.connect(_on_paste_version_err)
+
+    pbclient.paste(getpass.getuser(),
+                   "qute version info {}".format(qutebrowser.__version__),
+                   version(),
+                   private=True)
